@@ -16,6 +16,7 @@ from collections import namedtuple
 from functools import partial
 import itertools
 import logging
+import os
 import re
 
 import pytz
@@ -82,7 +83,7 @@ TronConfig = config_object_factory(
     'TronConfig',
     optional=[
          'working_dir',         # str
-         'syslog_address',      # str
+         'output_stream_dir',   # str
          'command_context',     # FrozenDict of str
          'ssh_options',         # ConchOptions
          'notification_options',# NotificationOptions or None
@@ -123,8 +124,7 @@ ConfigJob = config_object_factory(
         'node',                 # str
         'schedule',             # Config*Scheduler
         'actions',              # FrozenDict of ConfigAction
-    ],
-    [
+    ],[
         'queueing',             # bool
         'run_limit',            # int
         'all_nodes',            # bool
@@ -252,6 +252,9 @@ class Validator(object):
         if self.optional and in_dict is None:
             return None
 
+        if in_dict is None:
+            raise ConfigError("A %s is required." % self.type_name)
+
         shortcut_value = self.do_shortcut(in_dict)
         if shortcut_value:
             return shortcut_value
@@ -292,8 +295,9 @@ class Validator(object):
         if not missing_keys:
             return
 
+        keys = self.config_class.required_keys + self.config_class.optional_keys
         missing_key_str = ', '.join(missing_keys)
-        if 'name' in self.config_class.required_keys and 'name' in in_dict:
+        if 'name' in keys and 'name' in in_dict:
             raise ConfigError("%s %s is missing options: %s" % (
                     self.type_name, in_dict['name'], missing_key_str)
             )
@@ -358,15 +362,31 @@ class ValidatorWithNamedPath(Validator):
         return valid_input
 
 
+# TODO: remove in 0.5
 def valid_working_dir(wd):
     """Given a working directory or None, return a valid working directory.
     If wd=None the mcp will attempt to use a default.
     """
-    return valid_str('working_dir', wd, optional=True)
+    log.warning("working_dir is no longer supported. "
+        "Use output_stream_dir to set the directory for stdout/stderr files.")
 
 
-def valid_syslog(syslog):
-    return valid_str('syslog', syslog, optional=True)
+def valid_output_stream_dir(output_dir):
+    """Returns a valid string for the output directory, or raises ConfigError
+    if the output_dir is not valid.
+    """
+    output_dir = valid_str('output_stream_dir', output_dir, optional=True)
+    if not output_dir:
+        return
+
+    if not os.path.isdir(output_dir):
+        msg = "output_stream_dir '%s' is not a directory"
+        raise ConfigError(msg % output_dir)
+
+    if not os.access(output_dir, os.W_OK):
+        raise ConfigError("output_stream_dir '%s' is not writable" % output_dir)
+
+    return output_dir
 
 
 def valid_command_context(context):
@@ -386,6 +406,7 @@ def valid_time_zone(tz):
 class ValidateSSHOptions(Validator):
     """Validate SSH options."""
     config_class =              ConfigSSHOptions
+    optional =                  True
     defaults = {
         'agent':                False,
         'identities':           ()
@@ -406,6 +427,7 @@ class ValidateNotificationOptions(Validator):
 valid_notification_options = ValidateNotificationOptions()
 
 
+# TODO: remove in 0.5
 def normalize_node(node):
     """Given a node value from a config, determine if it's the node's name, the
     node's value, or actually a node pool. The former case is the "new style"
@@ -442,7 +464,10 @@ class ValidateNodePool(Validator):
     def cast(self, node_pool):
         if isinstance(node_pool, list):
             node_pool = dict(nodes=node_pool)
-        node_pool['nodes'] = [normalize_node(node) for node in node_pool['nodes']]
+        if 'nodes' in node_pool:
+            node_pool['nodes'] = [
+                normalize_node(node) for node in node_pool['nodes']
+            ]
         return node_pool
 
     def set_defaults(self, node_pool):
@@ -476,6 +501,9 @@ class ValidateAction(ValidatorWithNamedPath):
 
         # string identifier
         if isinstance(old_requires, basestring):
+            log.warn("Require without a list is deprecated. "
+                "You should update requires for %s %s" %
+                (path_name, action['name']))
             old_requires = [old_requires]
 
         # pointer
@@ -632,7 +660,7 @@ class ValidateConfig(Validator):
     config_class =              TronConfig
     defaults = {
         'working_dir':          None,
-        'syslog_address':       None,
+        'output_stream_dir':    None,
         'command_context':      None,
         'ssh_options':          valid_ssh_options({}),
         'notification_options': None,
@@ -643,8 +671,8 @@ class ValidateConfig(Validator):
         'services':             (),
     }
     validators = {
+        'output_stream_dir':    valid_output_stream_dir,
         'working_dir':          valid_working_dir,
-        'syslog_address':       valid_syslog,
         'command_context':      valid_command_context,
         'ssh_options':          valid_ssh_options,
         'notification_options': valid_notification_options,
@@ -690,6 +718,7 @@ class ValidateConfig(Validator):
         parse_sub_config('services',    valid_service,      job_service_names)
 
         self.validate_node_names(config, node_names)
+        self.validate_node_pool_nodes(config)
 
     def validate_node_names(self, config, node_names):
         """Validate that any node/node_pool name that were used are configured
@@ -709,6 +738,22 @@ class ValidateConfig(Validator):
                 raise ConfigError("Unknown node %s configured for %s %s" % (
                     task.node, task.__class__.__name__, task.name))
 
+    def validate_node_pool_nodes(self, config):
+        """Validate that each node in a node_pool is in fact a node, and not
+        another pool.
+        """
+        # TODO: this can be cleaned up after 0.5
+        for node_pool in config['node_pools'].itervalues():
+            for node_name in node_pool.nodes:
+                node = config['nodes'].get(node_name)
+                if node:
+                    continue
+                raise ConfigError(
+                    "NodePool %s contains another NodePool %s. " % (
+                        node_pool.name,
+                        node_name
+                    )
+                )
 
 
 valid_config = ValidateConfig()
