@@ -58,7 +58,6 @@ class JobRun(object):
                 ('is_queued',       all,    False),
                 ('is_cancelled',    all,    False),
                 ('is_skipped',      all,    False),
-                ('is_done',         all,    False),
                 ('check_state',     all,    True),
                 ('cancel',          all,    True),
                 ('succeed',         all,    True),
@@ -290,6 +289,12 @@ class JobRun(object):
             return True
         return all(r.is_done for r in self.action_runs)
 
+    @property
+    def is_done(self):
+        if self.is_failure:
+            return True
+        return all(r.is_done for r in self.action_runs_with_cleanup)
+
     @property
     def cleanup_job_status(self):
         """Provide 'SUCCESS' or 'FAILURE' to a cleanup action context based on
@@ -471,9 +476,7 @@ class Job(object):
         def choose(prev, nxt):
             if ((prev and prev.is_running) or
                 (node and nxt.node != node) or
-                nxt.is_success or
-                nxt.is_failure or
-                nxt.is_cancelled or
+                nxt.is_done or
                 nxt.is_unknown):
                 return prev
             else:
