@@ -3,7 +3,6 @@ import logging
 import logging.handlers
 import os
 import shutil
-import socket
 import time
 import yaml
 
@@ -222,6 +221,8 @@ class MasterControlProgram(object):
         # Root command context
         self.context = context or command_context.CommandContext()
 
+        self.output_stream_dir = None
+        self.working_dir = working_dir
         self.monitor = None
 
         # Time zone of the system clock
@@ -236,9 +237,6 @@ class MasterControlProgram(object):
         # Control writing of the state file
         self.state_handler = StateHandler(self, working_dir)
 
-        root = logging.getLogger('')
-        self.base_logging_handlers = list(root.handlers)
-
     ### CONFIGURATION ###
 
     def live_reconfig(self):
@@ -290,8 +288,7 @@ class MasterControlProgram(object):
         edit the config may not have the same environment as the live
         trond machine.
         """
-        self._apply_working_directory(conf.working_dir)
-        self._apply_loggers(conf.syslog_address)
+        self._apply_output_stream_directory(conf.output_stream_dir)
 
         if not skip_env_dependent:
             ssh_options = self._ssh_options_from_config(conf.ssh_options)
@@ -307,60 +304,9 @@ class MasterControlProgram(object):
         self._apply_services(conf.services)
         self._apply_notification_options(conf.notification_options)
 
-    def _apply_working_directory(self, wd):
-        """Apply working directory.  If wd=None try os.environ['TMPDIR'].
-        If that doesn't exist, use /tmp.
-        """
-        if self.state_handler.working_dir:
-            if self.state_handler.working_dir != wd:
-                log.warn('trond must be restarted for changes to the working'
-                         ' directory to take effect.')
-            return
-
-        if not wd:
-            if 'TMPDIR' in os.environ:
-                wd = os.environ['TMPDIR']
-            else:
-                wd = '/tmp'
-
-        if not os.path.isdir(wd):
-            raise ConfigApplyError("Specified working directory '%s' is not"
-                                   " a directory" % wd)
-
-        if not os.access(wd, os.W_OK):
-            raise ConfigApplyError("Specified working directory '%s' is not"
-                                   " writable" % wd)
-
-        self.state_handler.working_dir = wd
-
-    def _apply_loggers(self, syslog_address):
-        root = logging.getLogger('')
-        handlers_to_be_removed = set(h for h in root.handlers
-                                     if h not in self.base_logging_handlers)
-
-        # Only change handlers if they will actually be different from the old
-        # handlers
-        new_handlers = []
-        if syslog_address:
-            if not isinstance(syslog_address, basestring):
-                self.syslog_address = tuple(self.syslog_address)
-
-            try:
-                h = logging.handlers.SysLogHandler(syslog_address)
-                fmt_str = "tron[%(process)d]: %(message)s"
-                h.setFormatter(logging.Formatter(fmt_str))
-                new_handlers.append(h)
-            except socket.error:
-                raise ConfigError('%s is not a valid syslog address' %
-                                  syslog_address)
-
-        for h in handlers_to_be_removed:
-            log.info('Removing logging handler %s', h)
-            root.removeHandler(h)
-
-        for h in new_handlers:
-            log.info('Adding logging handler %s', h)
-            root.addHandler(h)
+    def _apply_output_stream_directory(self, output_stream_dir):
+        """Apply the output stream directory."""
+        self.output_stream_dir = output_stream_dir or self.working_dir
 
     def _ssh_options_from_config(self, ssh_conf):
         ssh_options = ConchOptions()
@@ -480,7 +426,7 @@ class MasterControlProgram(object):
         job.event_recorder.set_parent(self.event_recorder)
 
         # make the directory for output
-        job.setup_job_dir(self.state_handler.working_dir)
+        job.setup_job_dir(self.output_stream_dir)
 
         # Tell the job to call store_state() whenever its state changes.
         # job isn't actaully a StateMachine object, but its interface tries
@@ -593,8 +539,6 @@ class MasterControlProgram(object):
             else:
                 log.warning("Job name %s from state file unknown", name)
 
-                self.state_handler.restore_job(self.jobs[name], data[name])
-
         for name in data['services'].iterkeys():
             if name in self.services:
                 state_load_count += 1
