@@ -73,7 +73,7 @@ class NodePool(object):
     def from_config(cls, node_pool_config, nodes):
         return cls(
             name=node_pool_config.name,
-            nodes=[nodes[n] for n in nodes]
+            nodes=[nodes[n] for n in node_pool_config.nodes]
         )
 
     def __eq__(self, other):
@@ -83,9 +83,11 @@ class NodePool(object):
         return not self == other
 
     def next(self):
-        return self.nodes[random.randrange(len(self.nodes))]
+        """Return a random node from the pool."""
+        return random.choice(self.nodes)
 
     def next_round_robin(self):
+        """Return the next node cycling in a consistent order."""
         if not self.iter:
             self.iter = itertools.cycle(self.nodes)
 
@@ -95,8 +97,7 @@ class NodePool(object):
         for node in self.nodes:
             if node.hostname == value:
                 return node
-        else:
-            raise KeyError(value)
+        raise KeyError(value)
 
     def repr_data(self):
         """Returns a dict which is an external view of this object."""
@@ -173,7 +174,7 @@ class Node(object):
         """We want to introduce some amount of delay to node exec commands
 
         We see issues where a service may have many instances, and they all
-        start at once, and their monitor steps are syncronized for ever. This
+        start at once, and their monitor steps are blocked for ever. This
         is bad.
         """
         outstanding_runs = len(self.run_states)
